-- EasyMail
-- Written by Thott
-- Modified to be standalone and not dependant of Cosmos by Ramdor and LedMirage


function HookFunction(orig,new,htype)
  if(not htype) then
    htype = "before";
  end
  -- DebugPrint("Hooking "..orig.." to "..new..", type "..htype);
  if(not Hooks) then
    Hooks = {};
  end
  if(not Hooks[orig]) then
    Hooks[orig] = {};
    Hooks[orig].before = {};
    Hooks[orig].before.n = 0;
    Hooks[orig].after = {};
    Hooks[orig].after.n = 0;
    Hooks[orig].hide = {};
    Hooks[orig].hide.n = 0;
    Hooks[orig].orig = getglobal(orig);
  else
    for key,value in Hooks[orig] do
      if(key == new) then
	DebugPrint("already hooked "..new..", skipping");
	return;
      end
    end
  end
  -- intentionally will error if bad type is passed
  push(Hooks[orig][htype],getglobal(new));
  setglobal(orig,function(...) HookHandler(orig,arg); end);
end

function push(t,v)
  if(not t or not t.n) then
    dbanner("Bad table passed to push");
    return nil;
  end
  t.n = t.n+1;
  t[t.n] = v;
end

function HookHandler(name,arg)
  --dprint("HookHandler ",name);
  local called = false;
  local continue = true;
  local retval;
  for key,value in Hooks[name].hide do
    if(type(value) == "function") then
      --dprint("calling before ",name);
      if(not value(unpack(arg))) then
        continue = false;
      end
      called = true;
    end
  end
  --if(not continue) then
  --  dprint("hide returned false, aborting call to ",name);
  --  return;
  --end
  for key,value in Hooks[name].before do
    if(type(value) == "function") then
      --dprint("calling before ",name);
      value(unpack(arg));
      called = true;
    end
  end
  --dprint("calling original ",name);
  retval = Hooks[name].orig(unpack(arg));
  for key,value in Hooks[name].after do
    if(type(value) == "function") then
      --dprint("calling after ",name);
      value(unpack(arg));
      called = true;
    end
  end
  --if(not called) then
  --  dprint("no hooks left for ",name,", clearing");
  --  setglobal(name,Hooks[name].orig);
  --  Hooks[name] = nil;
  --end
  return retval;
end

function EasyMail_Schedule_OnLoad()
  EasyMail.Todo = {};
  EasyMail.Todo.n = 0;
end


function EasyMail_Schedule(when,handler,...)
  local todo = {};
  todo.time = when + GetTime();
  todo.handler = handler;
  todo.args = arg;
  local i = 1;
  while(EasyMail.Todo[i] and
  	EasyMail.Todo[i].time < todo.time) do
    i = i + 1;
  end
  table.insert(EasyMail.Todo,i,todo);
  --if(EasyMail.Debug) then
  --  dprint("scheduled ",handler," in ",when," seconds");
  --end
end


function EasyMail_Schedule_OnUpdate()
  while(EasyMail.Todo[1] and 
	EasyMail.Todo[1].time <= GetTime()) do
    local todo = table.remove(EasyMail.Todo,1);
    if(todo.args) then
      todo.handler(unpack(todo.args));
    else
      todo.handler();
    end
  end
end


function EasyMail_OnLoad()
  HookFunction("SendMailFrame_SendMail","EasyMail_SendMail","before");
  HookFunction("MailFrameTab_OnClick","EasyMail_MailFrameTab","after");
  HookFunction("SendMailFrame_Update","EasyMail_SendMailUpdate","after");
  RegisterForSave("EasyMail_LastMailed");
end


function EasyMail_SendMail()
  EasyMail_LastMailed = SendMailNameEditBox:GetText();
  EasyMail_Schedule(0.1,EasyMail_MailFrameTab);
  EasyMail_Schedule(1.0,EasyMail_MailFrameTab);
end


function EasyMail_MailFrameTab(tab)
  if(EasyMail_LastMailed) then
    local text = SendMailNameEditBox:GetText();
    if(not text or text == "") then
      SendMailNameEditBox:SetText(EasyMail_LastMailed);
    end
  end
end


function EasyMail_SendMailUpdate()
  local itemName,texture,count = GetSendMailItem();
  --dprint("itemName: ",itemName);
  if(itemName) then
    local subject = SendMailSubjectEditBox:GetText();
    if(not subject or subject == "") then
      if(count and count > 1) then
	SendMailSubjectEditBox:SetText(itemName.." x "..count);
      else
	SendMailSubjectEditBox:SetText(itemName);
      end
    end
  end
end


