-- Configuration variables
Nurfed_Default_Offset_Hour = 1;		-- this is the hour offset applied to server time
Nurfed_Default_Offset_Minute = 1;	-- this is the minute offset applied to server time
Nurfed_Default_MilitaryTime = 0;	-- set this to something other than nil to use 24-hour display
Nurfed_Default_TimeStamps = 1;		-- set this to nil to turn off Chat Timestamps

-- Initial setup of action button gap slider settings
Nurfed_HourOffsetMax = 12;
Nurfed_HourOffsetMin = -12;
Nurfed_HourOffsetStep = 1;

Nurfed_MinuteOffsetMax = 59;
Nurfed_MinuteOffsetMin = 0;
Nurfed_MinuteOffsetStep = 1;

-- Localizable strings
TIME_PLAYED_SESSION = "Time played this session: %s"; -- The amount of time played this session
CLOCK_TIME_DAY = "%d day";
CLOCK_TIME_HOUR = "%d hr";
CLOCK_TIME_MINUTE = "%d min";
CLOCK_TIME_SECOND = "%d sec";
EXP_PER_HOUR_LEVEL = "XP per hour this level: %.2f";
EXP_PER_HOUR_SESSION = "XP per hour this session: %.2f";
EXP_PERCENT = "XP: %.2f%%";
TIME_TO_LEVEL_LEVEL = "This level's rate: %s";
TIME_TO_LEVEL_SESSION = "This session's rate: %s";
TIME_INFINITE = "infinite";
RESET_CLOCK = "Right Click Clock to Reset";
MOVE_CLOCK = "Unlock UI and Left Click Clock to Move";
PLAYER_XP = "Current XP: %s";

-- jlr
local jlr_lastmin = 0;
local jlr_time = 0;
local jlr_TIME_TWELVEHOURAM = "%d:%02d:%02d AM"; -- %d = hour, %d = minute
local jlr_TIME_TWELVEHOURPM = "%d:%02d:%02d PM"; -- %d = hour, %d = minute
-- jlr

-- Constants
CLOCK_UPDATE_RATE = 0.1;

-- Local variables
local TotalTimePlayed = 0;
local LevelTimePlayed = 0;
local SessionTimePlayed = 0;
local ElapsedSinceLastPlayedMessage = 0;
local NeedPlayedMessage = 1;

local localInitialXP;
local localSessionXP = 0;
local localRolloverXP = 0;

local localInCombat;

-- OnFoo functions
function Clock_OnLoad()
	this:RegisterEvent("TIME_PLAYED_MSG");
	this:RegisterEvent("PLAYER_LEVEL_UP");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_XP_UPDATE");
	this:RegisterEvent("UNIT_HEALTH");
	this:RegisterEvent("UNIT_MANA");
	this:RegisterEvent("PLAYER_ENTER_COMBAT");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("VARIABLES_LOADED");
	
	Nurfed_ClockFrame.TimeSinceLastUpdate = 0;
end



-- ** EXPERIENCE BAR FUNCTIONS ** --

function Nurfed_PlayerFrameExpBarOnLoad ()
	-- Experience bar registrations
	this:RegisterEvent("PLAYER_XP_UPDATE");
	this:RegisterEvent("PLAYER_LEVEL_UP");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
end

function Nurfed_PlayerFrameExpBarOnEvent (event)
	-- Setup our experience bar when the player logs in, when experience changes, or when the player levels up
	if (event == "PLAYER_XP_UPDATE" or event == "PLAYER_LEVEL_UP" or event == "PLAYER_ENTERING_WORLD") then
		local currXP = UnitXP("player");
		local nextXP = UnitXPMax("player");
		this:SetMinMaxValues(min(0, currXP), nextXP);
		this:SetValue(currXP);
		return;
	end
end

function Nurfed_ShowPlayerExp()
	local currXP = UnitXP("player");
	local nextXP = UnitXPMax("player");
	local restXP = GetXPExhaustion();
	local percentXP = format(TEXT(EXP_PERCENT),(currXP / nextXP) * 100);
	Nurfed_XPText:SetText(percentXP);

	if(restXP == nil) then
		local str = format("%s / %s", currXP, nextXP);
		Nurfed_ExpAmount = (str);
		Nurfed_PlayerXPBar:SetStatusBarColor(0.58, 0.0, 0.55, 1.0);
	else
		-- Divide the rested # in half because
		-- (a) that's the exp you really end up getting
		-- (b) it isn't such an absurdly huge number taking up a lot of space on the UI.

		local str = format("%s / %s (+%s)", currXP, nextXP, (tonumber(restXP) / 2));
		Nurfed_ExpAmount = (str);
		Nurfed_PlayerXPBar:SetStatusBarColor(0.0, 0.39, 1.0, 1.0);
	end
end

function Clock_OnUpdate(arg1)
	SessionTimePlayed = SessionTimePlayed + arg1;
	ElapsedSinceLastPlayedMessage = ElapsedSinceLastPlayedMessage + arg1;
	Nurfed_ClockFrame.TimeSinceLastUpdate = Nurfed_ClockFrame.TimeSinceLastUpdate + arg1;
	if( Nurfed_ClockFrame.TimeSinceLastUpdate > CLOCK_UPDATE_RATE ) then
		Nurfed_ClockText:SetText(Clock_GetTimeText());
		if( GameTooltip:IsOwned(this) ) then
			Clock_SetTooltip();
		end
		Nurfed_ClockFrame.TimeSinceLastUpdate = 0;
	end
end

function Clock_OnEvent()
	if( event == "TIME_PLAYED_MSG" ) then
		TotalTimePlayed = arg1;
		LevelTimePlayed = arg2;
		ElapsedSinceLastPlayedMessage = 0;
		NeedPlayedMessage = 0;
		
		-- Sync up all of the times to the session time; this makes
		-- the tooltip look nicer as everything changes all at once
		local fraction = SessionTimePlayed - floor(SessionTimePlayed);
		TotalTimePlayed = floor(TotalTimePlayed) + fraction;
		LevelTimePlayed = floor(LevelTimePlayed) + fraction;
		
		if( GameTooltip:IsOwned(this) ) then
			Clock_SetTooltip();
		end
	elseif( event == "PLAYER_LEVEL_UP" ) then
		LevelTimePlayed = SessionTimePlayed - floor(SessionTimePlayed);
		localRolloverXP = localSessionXP;
		localInitialXP = 0;
		if( GameTooltip:IsOwned(this) ) then
			Clock_SetTooltip();
		end
		Nurfed_ShowPlayerExp();
	elseif( event == "PLAYER_ENTERING_WORLD" ) then
		if( not localInitialXP ) then
			localInitialXP = UnitXP("player");
			--[[
			localLastPosition = { };
			localLastPosition.x, localLastPosition.y, localLastPosition.z = Clock_ParsePosition(GetCurrentPosition());
			if( not localLastPosition.z ) then
				localLastPosition = nil;
			end
			]]
		end
		Nurfed_ShowPlayerExp();
	elseif( event == "PLAYER_XP_UPDATE" ) then
		if( localInitialXP ) then
			localSessionXP = UnitXP("player") - localInitialXP + localRolloverXP;
		end
		Nurfed_ShowPlayerExp();
	elseif( event == "PLAYER_ENTER_COMBAT" ) then
		localInCombat = 1;
	elseif( event == "PLAYER_LEAVE_COMBAT" ) then
		localInCombat = nil;
	end
	if (event == "VARIABLES_LOADED") then
		if(NURFED_GENERAL == 1) then
			if (Nurfed_RetrieveSaved("offsethour") == nil) then
				Nurfed_Offset_Hour = Nurfed_Default_Offset_Hour;
			else
				Nurfed_Offset_Hour = Nurfed_RetrieveSaved("offsethour");
			end
			if (Nurfed_RetrieveSaved("offsetminute") == nil) then
				Nurfed_Offset_Minute = Nurfed_Default_Offset_Minute;
			else
				Nurfed_Offset_Minute = Nurfed_RetrieveSaved("offsetminute");
			end
			if (Nurfed_RetrieveSaved("militarytime") == nil) then
				Nurfed_MilitaryTime = Nurfed_Default_MilitaryTime;
			else
				Nurfed_MilitaryTime = Nurfed_RetrieveSaved("militarytime");
			end
			if (Nurfed_RetrieveSaved("timestamps") == nil) then
				Nurfed_TimeStamps = Nurfed_Default_TimeStamps;
			else
				Nurfed_TimeStamps = Nurfed_RetrieveSaved("timestamps");
			end

			Nurfed_RegisterOption(100, "category", nil, nil, nil, "Clock", "Settings for the Clock and Timestamps.");
			Nurfed_RegisterOption(101, "header", nil, nil, nil, "Clock Options", "Options relating to the Clock.");
			Nurfed_RegisterOption(102, "slider", Nurfed_Offset_Hour, "offsethour", Nurfed_ClockUpdate, "Offset Clock Hours", "Number of hours to offset\nthe in-game clock", Nurfed_HourOffsetMin, Nurfed_HourOffsetMax, Nurfed_HourOffsetStep, " Hr.");
			Nurfed_RegisterOption(103, "slider", Nurfed_Offset_Minute, "offsetminute", Nurfed_ClockUpdate,"Offset Clock Minutes", "Number of minutes to offset\nthe in-game clock", Nurfed_MinuteOffsetMin, Nurfed_MinuteOffsetMax, Nurfed_MinuteOffsetStep, " Min.");
			Nurfed_RegisterOption(104, "check", Nurfed_MilitaryTime, "militarytime", Nurfed_ClockUpdate, "Enable Military Time", "Military Time 24 hour.");
			Nurfed_RegisterOption(105, "check", Nurfed_TimeStamps, "timestamps", Nurfed_ClockUpdate, "Enable Time Stamps", "Chat Time Stamps.");
		else
			if (Nurfed_Offset_Hour == nil) then
				Nurfed_Offset_Hour = Nurfed_Default_Offset_Hour;
			end
			if (Nurfed_Offset_Minute == nil) then
				Nurfed_Offset_Minute = Nurfed_Default_Offset_Minute;
			end
			if (Nurfed_MilitaryTime == nil) then
				Nurfed_MilitaryTime = Nurfed_Default_MilitaryTime;
			end
			if (Nurfed_TimeStamps == nil) then
				Nurfed_TimeStamps = Nurfed_Default_TimeStamps;
			end
		end
	end
end

function Nurfed_ClockUpdate(init)
	if (init == nil) then
		init = 0;
	end
	if (Nurfed_RetrieveOption ~= nil) then
		if (Nurfed_RetrieveOption(102) ~= nil) then
			Nurfed_Offset_Hour = Nurfed_RetrieveOption(102)[NURFED_VALUE];
		end
		if (Nurfed_RetrieveOption(103) ~= nil) then
			Nurfed_Offset_Minute = Nurfed_RetrieveOption(103)[NURFED_VALUE];
		end
		if (Nurfed_RetrieveOption(104) ~= nil) then
			Nurfed_MilitaryTime = Nurfed_RetrieveOption(104)[NURFED_VALUE];
		end
		if (Nurfed_RetrieveOption(105) ~= nil) then
			Nurfed_TimeStamps = Nurfed_RetrieveOption(105)[NURFED_VALUE];
		end
	end
end

function ClockText_OnEnter()
	GameTooltip:SetOwner(Nurfed_ClockFrame, "ANCHOR_NONE");
	GameTooltip:SetPoint("TOP", "UIParent", "TOP", 0, -32);
	if( NeedPlayedMessage == 1 ) then
		RequestTimePlayed();
	end
	Clock_SetTooltip();
end

-- Helper functions
function Clock_GetTimeText()
	local hour, minute = GetGameTime();
	local pm;
	
	-- jlr
	if (minute ~= jlr_lastmin) then
	  -- We just flipped to a new minute value
	  jlr_time = GetTime();
	  jlr_lastmin = minute;
	 end
	-- jlr
	
	hour = hour + Nurfed_Offset_Hour;
	minute = minute + Nurfed_Offset_Minute;
	if( minute > 59 ) then
		minute = minute - 60;
		hour = hour + 1
	elseif( minute < 0 ) then
		minute = 60 + minute;
		hour = hour - 1;
	end
	if( hour > 23 ) then
		hour = hour - 24;
	elseif( hour < 0 ) then
		hour = 24 + hour;
	end
	
	if( Nurfed_MilitaryTime == 1 ) then
		return format(TEXT(TIME_TWENTYFOURHOURS), hour, minute);
	else
		if( hour >= 12 ) then
			pm = 1;
			hour = hour - 12;
		else
			pm = 0;
		end
		if( hour == 0 ) then
			hour = 12;
		end
		if( pm == 1 ) then
			return format(TEXT(jlr_TIME_TWELVEHOURPM), hour, minute, GetTime() - jlr_time);
		else
			return format(TEXT(jlr_TIME_TWELVEHOURAM), hour, minute, GetTime() - jlr_time);
		end
	end
end

local function Clock_FormatPart(fmt, val)
	local part;
	
	part = format(TEXT(fmt), val);
	if( val ~= 1 ) then
		part = part.."s";
	end
	
	return part;
end

function Clock_FormatTime(time)
	local d, h, m, s;
	local text = "";
	local skip = 1;

	d, h, m, s = ChatFrame_TimeBreakDown(time);
	if( d > 0 ) then
		text = text..Clock_FormatPart(CLOCK_TIME_DAY, d)..", ";
		skip = 0;
	end
	if( (skip == 0) or (h > 0) ) then
		text = text..Clock_FormatPart(CLOCK_TIME_HOUR, h)..", ";
		skip = 0;
	end
	if( (skip == 0) or (m > 0) ) then
		text = text..Clock_FormatPart(CLOCK_TIME_MINUTE, m)..", ";
		skip = 0;
	end
	if( (skip == 0) or (s > 0) ) then
		text = text..Clock_FormatPart(CLOCK_TIME_SECOND, s);
	end
	
	return text;
end

function Clock_SetTooltip()
	local total, level, session;
	local xpPerHourLevel, xpPerHourSession;
	local xpTotal, xpCurrent, xpToLevel;
	local text;
	
	total = format(TEXT(TIME_PLAYED_TOTAL), Clock_FormatTime(TotalTimePlayed + ElapsedSinceLastPlayedMessage));
	level = format(TEXT(TIME_PLAYED_LEVEL), Clock_FormatTime(LevelTimePlayed + ElapsedSinceLastPlayedMessage));
	session = format(TEXT(TIME_PLAYED_SESSION), Clock_FormatTime(SessionTimePlayed));
	
	if( NeedPlayedMessage == 1 ) then
		text = session;
	else
		text = total.."\n"..level.."\n"..session;
	end
	
	if( (LevelTimePlayed + ElapsedSinceLastPlayedMessage > 0) or SessionTimePlayed > 0 ) then
		text = text.."\n";
	end
	if( LevelTimePlayed + ElapsedSinceLastPlayedMessage > 0 ) then
		xpPerHourLevel = UnitXP("player") / ((LevelTimePlayed + ElapsedSinceLastPlayedMessage) / 3600);
		text = text.."\n"..format(TEXT(EXP_PER_HOUR_LEVEL), xpPerHourLevel);
	else
		xpPerHourLevel = 0;
	end
	if( SessionTimePlayed > 0 ) then
		xpPerHourSession = localSessionXP / (SessionTimePlayed / 3600);
		text = text.."\n"..format(TEXT(EXP_PER_HOUR_SESSION), xpPerHourSession);
	else
		xpPerHourSession = 0;
	end
	
	xpTotal = UnitXPMax("player");
	xpCurrent = UnitXP("player");
	if( xpCurrent < xpTotal ) then
		xpToLevel = xpTotal - xpCurrent;
		--text = text.."\n"..format(TEXT(EXP_TO_LEVEL), xpToLevel, (xpToLevel / xpTotal) * 100);
		if( xpPerHourLevel > 0 ) then
			text = text.."\n"..format(TEXT(TIME_TO_LEVEL_LEVEL), Clock_FormatTime((xpToLevel / xpPerHourLevel) * 3600));
		else
			text = text.."\n"..format(TEXT(TIME_TO_LEVEL_LEVEL), TEXT(TIME_INFINITE));
		end
		if( xpPerHourSession > 0 ) then
			text = text.."\n"..format(TEXT(TIME_TO_LEVEL_SESSION), Clock_FormatTime((xpToLevel / xpPerHourSession) * 3600));
		else
			text = text.."\n"..format(TEXT(TIME_TO_LEVEL_SESSION), TEXT(TIME_INFINITE));
		end
	end
	text = text.."\n\n"..format(TEXT(RESET_CLOCK));
	text = text.."\n"..format(TEXT(MOVE_CLOCK));
	text = text.."\n\n"..format(TEXT(PLAYER_XP), Nurfed_ExpAmount);
	
	GameTooltip:SetText(text);
end

function Nurfed_Latency()
	local bandwidthIn, bandwidthOut, latency = GetNetStats();
	Nurfed_LatencyText:SetText((latency).." ms");
	if( latency > 500 ) then
		Nurfed_LatencyText:SetTextColor(1, 0, 0);
	elseif (latency > 200 ) then
		Nurfed_LatencyText:SetTextColor(1, 1, 0);
	else
		Nurfed_LatencyText:SetTextColor(0, 1, 0);
	end
end

function ClockReset()
	TotalTimePlayed = 0;
	LevelTimePlayed = 0;
	SessionTimePlayed = 0;
	ElapsedSinceLastPlayedMessage = 0;
	NeedPlayedMessage = 1;

	localInitialXP = UnitXP("player");
	localSessionXP = 0;
	localRolloverXP = 0;

	localInCombat = nil;
end

function Nurfed_ClockMouseUp(button)
	if ( button == "RightButton" ) then
		ClockReset();
		return;
	end
	Nurfed_ClockFrame:StopMovingOrSizing();
end

function Nurfed_ClockMouseDown(button)
	if ( button == "LeftButton" and NURFED_LOCKALL == 0) then
		Nurfed_ClockFrame:StartMoving();
	end
end

--Chat Timestamps
local Original_ChatFrame_OnEvent;

function ChatTimestamp_OnLoad()
	-- Hook ChatFrame_OnEvent so we can hook AddMessage
	Original_ChatFrame_OnEvent = ChatFrame_OnEvent;
	ChatFrame_OnEvent = ChatTimestamp_ChatFrame_OnEvent;
end

function ChatTimestamp_ChatFrame_OnEvent(event)
	Original_ChatFrame_OnEvent(event); --call the real ChatFrame_OnEvent function
	--if we haven't already done so, hook the AddMessage function
	if(not this.Original_AddMessage) then
		this.Original_AddMessage = this.AddMessage;
		this.AddMessage = ChatTimestamp_AddMessage;
	end
end

function ChatTimestamp_AddMessage(this, msg, r, g, b, id)
	local newmsg = format("[%s] %s",Clock_GetTimeText(), msg); --format the output
	if(Nurfed_TimeStamps == 1) then
		this:Original_AddMessage(newmsg, r, g, b, id); --call the real AddMessage function
	else
		this:Original_AddMessage(msg, r, g, b, id); --call the real AddMessage function
	end
end
