local function Error( msg )
	Print( msg, 1.0, 0.2, 0.2 );
end

local function Print(msg, r, g, b, frame)
	if (not r) then r = 1.0; end
	if (not g) then g = 1.0; end
	if (not b) then b = 1.0; end
	if ( frame ) then
		frame:AddMessage(msg,r,g,b);
	else
		if ( DEFAULT_CHAT_FRAME ) then
			DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
		end
	end
end

-------------------------------------------------------------
BINDING_HEADER_WEAPONSETS = "Weapon Sets";
MAX_EQUIPMENT_SETS = 6;

EquipmentDelayedCommands = {};

local WeaponSet_ServerName = "";
local WeaponSet_PlayerName = "";

function ToggleWeaponSets()
	if( WeaponSets:IsVisible() ) then
		WeaponSets:Hide();
	else
		WeaponSets:Show();
	end
end

function GetWeaponSet( index )
	if( EquipmentSets[WeaponSet_ServerName] == nil or
		EquipmentSets[WeaponSet_ServerName][WeaponSet_PlayerName] == nil ) then
		return nil
	end
	return EquipmentSets[WeaponSet_ServerName][WeaponSet_PlayerName][index];
end

function CreateWeaponSet( index )
	if( EquipmentSets[WeaponSet_ServerName] == nil ) then
		EquipmentSets[WeaponSet_ServerName] = {};
	end
	if( EquipmentSets[WeaponSet_ServerName][WeaponSet_PlayerName] == nil ) then
		EquipmentSets[WeaponSet_ServerName][WeaponSet_PlayerName] = {};
	end
	if( EquipmentSets[WeaponSet_ServerName][WeaponSet_PlayerName][index] == nil ) then
		EquipmentSets[WeaponSet_ServerName][WeaponSet_PlayerName][index] = {};
	end
end

function WeaponSetsUpdated()
	ResetEquipmentSets();
	local i = 1;
	local weaponset = getglobal( "WeaponSet"..i );
	while( weaponset ) do
		local main = getglobal( weaponset:GetName().."Main" ).item;
		local secondary = getglobal( weaponset:GetName().."Secondary" ).item;
		if( main == nil ) then main = ""; end
		if( secondary == nil ) then secondary = ""; end
		AddEquipmentSet( main, secondary, i );
		i = i + 1;
		weaponset = getglobal( "WeaponSet"..i );
	end
	WeaponSetCheckButtons_Update();
end

function DetermineCurrentWeaponSet()
	local i;
	for i = 1, MAX_EQUIPMENT_SETS, 1 do
		if( GetWeaponSet(i) ) then
			local main = GetInventoryItemName( CharacterMainHandSlot:GetID() );
			local secondary = GetInventoryItemName( CharacterSecondaryHandSlot:GetID() );
			if( main == nil ) then main = ""; end
			if( secondary == nil ) then secondary = ""; end
			if( GetWeaponSet(i)[0] == main and GetWeaponSet(i)[1] == secondary ) then
				return i;
			end
		end
	end
end

function WeaponSetButton_OnClick()
	if( CursorHasItem() ) then
		SetItemButtonTexture(this, ItemSource.texture);
		if( ItemSource.source == "CONTAINER" ) then
			this.item = GetItemName( ItemSource.bag, ItemSource.slot );
			PickupContainerItem( ItemSource.bag, ItemSource.slot );
		elseif( ItemSource.source == "INVENTORY" ) then
			this.item = GetInventoryItemName( ItemSource.slot );
			PickupInventoryItem( ItemSource.slot );
		end
		WeaponSetsUpdated();
	else

	end
end

function WeaponSetButton_OnEnter()
	if( this.item ) then
		GameTooltip:SetOwner(this, "ANCHOR_LEFT");
		local source, bag, slot = FindItemByName( this.item );
		if( source == "INVENTORY" ) then
			GameTooltip:SetInventoryItem("player", slot);
		elseif( source == "CONTAINER" ) then
			GameTooltip:SetBagItem( bag, slot );
		end
	end
end

function WeaponSetBinding_OnClick(button)
	if( WeaponSets.mappingKey ) then
		if( button == "LeftButton" or button == "RightButton" ) then
			WeaponSets.mappingKey = nil;
			WeaponSets_UpdateBindingText();
		else
			WeaponSets_OnKeyDown( button );
		end
	else
		WeaponSets.mappingKey = this;
		this:SetText( "Press a Key" );
	end
end

function WeaponSetCheckButtons_Update()
	local current = DetermineCurrentWeaponSet();
	local currentbutton = nil;
	if( current ) then
		currentbutton = getglobal( "WeaponSet"..current.."CheckButton" );
	end
	local i = 1;
	local button = getglobal( "WeaponSet"..i.."CheckButton" );
	while button do
		if( button == currentbutton ) then
			button:SetChecked( 1 );
		else
			button:SetChecked( nil );
		end
		i = i + 1;
		button = getglobal( "WeaponSet"..i.."CheckButton" );
	end
end

function WeaponSetCheckButton_OnClick()
	-- we want these to act like radio buttons
	-- uncheck any other buttons
	local i = 1;
	local main = getglobal( this:GetParent():GetName().."Main" ).item;
	local secondary = getglobal( this:GetParent():GetName().."Secondary" ).item;
	EquipWeapons( main, secondary );
	WeaponSetCheckButtons_Update();
end

function WeaponSets_OnEvent( event, arg1 )
	if( event == "VARIABLES_LOADED" and UnitName( "player" ) ) then
		WeaponSet_ServerName = GetCVar( "realmName" );
		WeaponSet_PlayerName = UnitName( "player" );
	elseif( event == "UNIT_NAME_UPDATE" and arg1 == "player" ) then
		WeaponSet_ServerName = GetCVar( "realmName" );
		WeaponSet_PlayerName = UnitName( "player" );
	end
	if( event == "ITEM_LOCK_CHANGED" and not arg1) then
		if( table.getn( EquipmentDelayedCommands ) > 0 ) then
			local delayed = table.remove( EquipmentDelayedCommands );
			local succeeded = delayed();
			if( succeeded ~= true ) then
				table.insert( EquipmentDelayedCommands, 1, delayed );
			end
		end
		WeaponSetCheckButtons_Update();
	end
end

function WeaponSets_GetWeaponTexture( name )
	source, bag, slot = FindItemByName( name );
	if( source == "INVENTORY" ) then
		return GetInventoryItemTexture("player", slot);
	elseif( source == "CONTAINER" ) then
		return GetContainerItemInfo(bag, slot);
	end
end

function WeaponSets_OnShow()
	WeaponSets.mappingKey = nil;
	WeaponSets_UpdateBindingText();

	local i = 1;
	local weaponset = getglobal( "WeaponSet"..i );
	while( weaponset ) do
		getglobal( "WeaponSet"..i.."Main" ).item = nil;
		SetItemButtonTexture( getglobal( "WeaponSet"..i.."Main" ), nil );
		getglobal( "WeaponSet"..i.."Secondary" ).item = nil;
		SetItemButtonTexture( getglobal( "WeaponSet"..i.."Secondary" ), nil );
		if( GetWeaponSet(i) ) then
			if( GetWeaponSet(i)[0] ~= nil and GetWeaponSet(i)[0] ~= "" ) then
				local texture = WeaponSets_GetWeaponTexture( GetWeaponSet(i)[0] )
				button = getglobal( "WeaponSet"..i.."Main" );
				SetItemButtonTexture(button, texture);
				button.item = GetWeaponSet(i)[0];
			end
			if( GetWeaponSet(i)[1] ~= nil and GetWeaponSet(i)[1] ~= "" ) then
				local texture = WeaponSets_GetWeaponTexture( GetWeaponSet(i)[1] )
				button = getglobal( "WeaponSet"..i.."Secondary" );
				SetItemButtonTexture(button, texture);
				button.item = GetWeaponSet(i)[1];
			end
		end
		i = i + 1;
		weaponset = getglobal( "WeaponSet"..i );
	end

	WeaponSetCheckButtons_Update();
end

function WeaponSets_UpdateBindingText()
	local i = 1;
	local button = getglobal( "WeaponSet"..i.."BindingButton" );
	while button do
		button:SetText("Not Bound");
		i = i + 1;
		button = getglobal( "WeaponSet"..i.."BindingButton" );
	end
end

function WeaponSets_OnKeyDown( button )
	if ( arg1 == "PRINTSCREEN" ) then
		Screenshot();
		return;
	end

	if( arg1 == "ESCAPE" ) then
		if( WeaponSets.mappingKey ) then
			WeaponSets.mappingKey = nil;
			WeaponSets_UpdateBindingText();
		else
			WeaponSets:Hide();
		end
		return;
	end

	-- Convert the mouse button names
	if ( button == "LeftButton" ) then
		button = "BUTTON1";
	elseif ( button == "RightButton" ) then
		button = "BUTTON2";
	elseif ( button == "MiddleButton" ) then
		button = "BUTTON3";
	elseif ( button == "Button4" ) then
		button = "BUTTON4"
	elseif ( button == "Button5" ) then
		button = "BUTTON5"
	end
	if ( WeaponSets.mappingKey ) then
		local keyPressed = arg1;
		if ( button ) then
			if ( button == "BUTTON1" or button == "BUTTON2" ) then
				return;
			end
			keyPressed = button;
		else
			keyPressed = arg1;
		end
		if ( keyPressed == "UNKNOWN" ) then
			return;
		end
		if ( keyPressed == "SHIFT" or keyPressed == "CTRL" or keyPressed == "ALT") then
			return;
		end
		if ( IsShiftKeyDown() ) then
			keyPressed = "SHIFT-"..keyPressed;
		end
		if ( IsControlKeyDown() ) then
			keyPressed = "CTRL-"..keyPressed;
		end
		if ( IsAltKeyDown() ) then
			keyPressed = "ALT-"..keyPressed;
		end

		-- clear the old binding if there is one
		local oldkey = GetBindingKey( WeaponSets.mappingKey:GetName() );
		if( oldkey ) then
			SetBinding( oldkey );
		end

		SetBinding( keyPressed, WeaponSets.mappingKey:GetName() );
		SaveBindings();
		WeaponSets_UpdateBindingText();
		WeaponSets.mappingKey = nil;
	end
end

-------------------------------------------------------------

ItemSource = {};
ItemSource.source = nil;

local OldPickupContainer = PickupContainerItem;
function PickupContainerItem( bag, slot )
	OldPickupContainer( bag, slot );
	if( CursorHasItem() ) then
		local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);
		ItemSource.source = "CONTAINER";
		ItemSource.texture = texture;
		ItemSource.bag = bag;
		ItemSource.slot = slot;
	end
end

local OldPickupInventory = PickupInventoryItem;
function PickupInventoryItem( index )
	OldPickupInventory( index );
	if( CursorHasItem() ) then
		ItemSource.source = "INVENTORY";
		ItemSource.texture = GetInventoryItemTexture("player", index);
		ItemSource.slot = index;
	end
end

function GetInventoryItemName( slot )
	local name = "";
	ClearToolTip();
	WeaponSetTooltip:SetOwner( getglobal("UIParent"), "ANCHOR_CURSOR" );
	if( WeaponSetTooltip:SetInventoryItem( "player", slot ) ) then
		if( WeaponSetTooltip:IsVisible() ) then
			WeaponSetTooltip:Hide();
			name = WeaponSetTooltipTextLeft1:GetText();
		end
	end
	return name;
end

function GetItemName( bag, slot )
	local name = "";
	ClearToolTip();
	WeaponSetTooltip:SetOwner( getglobal("UIParent"), "ANCHOR_CURSOR" );
	WeaponSetTooltip:SetBagItem( bag, slot );
	if( WeaponSetTooltip:IsVisible() ) then
		WeaponSetTooltip:Hide();
		name = WeaponSetTooltipTextLeft1:GetText();
	end
	return name;
end

function GetItemCount( bag, slot )
	local texture, itemCount, locked, quality = GetContainerItemInfo(bag, slot);
	return itemCount;
end

function FindEmptyBagSlots( howmany )
	local bags, slots = {}, {};
	local findingslot = 1;
	for bag=0, 4, 1 do
		if( GetContainerNumSlots( bag ) > 0 ) then
			for slot=1, GetContainerNumSlots( bag ), 1 do
				if( GetItemCount( bag, slot ) == nil ) then
					-- found a slot!
					bags[findingslot] = bag;
					slots[findingslot] = slot;
					findingslot = findingslot + 1;
					if( findingslot > howmany ) then
						return bags, slots;
					end
				end
			end
		end
	end
	return bags, slots
end

function FindEmptyBagSlot()
	local bags, slots;
	bags, slots = FindEmptyBagSlots( 1 );
	return bags[1], slots[1];
end

function FindItemByName( name )
	local bag, slot;
	-- look in inventory first
	for slot = 1, 18, 1 do
		if( GetInventoryItemName( slot ) == name ) then
			return "INVENTORY", nil, slot;
		end
	end

	for bag=0, 5, 1 do
		if( GetContainerNumSlots( bag ) > 0 ) then
			for slot=1, GetContainerNumSlots( bag ), 1 do
				if( GetItemName( bag, slot ) == name ) then
					return "CONTAINER", bag, slot;
				end
			end
		end
	end
	return "NOT FOUND", nil, nil;
end

function CountItemByName( name )
	local bag, slot, total;
	total = 0;
	for bag=0, 5, 1 do
		if( GetContainerNumSlots( bag ) > 0 ) then
			for slot=1, GetContainerNumSlots( bag ), 1 do
				if( GetItemName( bag, slot ) == name ) then
					total = total + GetItemCount( bag, slot );
				end
			end
		end
	end
	return total;
end

function ClearToolTip()
	for i=1, 15, 1 do
		getglobal("WeaponSetTooltipTextLeft"..i):SetText("");
		getglobal("WeaponSetTooltipTextRight"..i):SetText("");
	end
end

function PrintItemText( bag, slot )
	ClearToolTip();
	WeaponSetTooltip:SetOwner( getglobal("UIParent"), "ANCHOR_CURSOR" );
	WeaponSetTooltip:SetBagItem( bag, slot );
	if( WeaponSetTooltip:IsVisible() ) then
		Print( "Visible tooltip" );
		WeaponSetTooltip:Hide();
		for i=1, WeaponSetTooltip:NumLines(), 1 do
			show = 0;
			str = "Left"..i..": ";
			str2 = getglobal("WeaponSetTooltipTextLeft"..i):GetText();
			if( str2 ~= nil ) then
				show = 1;
				str = str..str2;
			end
			str = str.." Right"..i..": ";
			str2 = getglobal("WeaponSetTooltipTextRight"..i):GetText();
			if( str2 ~= nil ) then
				show = 1;
				str = str..str2;
			end
			if( show == 1 ) then Print( str ); end
		end
	end
end

EquipmentSets = {};

function ResetEquipmentSets()
	local i;
	for i = 1, MAX_EQUIPMENT_SETS, 1 do
		RemoveEquipmentSet( i );
	end
end

function AddEquipmentSet( main, secondary, index )
	-- find an empty set
	if( index == nil ) then
		local i;
		for i = 1, MAX_EQUIPMENT_SETS, 1 do
			if( GetWeaponSet(i) ) then
				if( GetWeaponSet(i)[0] == "" and GetWeaponSet(i)[1] == "" ) then
					empty = i;
					break;
				end
			else
				empty = i;
				break;
			end
		end
	end
	if( index == nil ) then
		Error( "Only "..MAX_EQUIPMENT_SETS.." equipment sets are allowed" );
		return;
	end
	CreateWeaponSet( index );
	if( main == nil ) then main = GetInventoryItemName( CharacterMainHandSlot:GetID() ); end
	if( secondary == nil ) then secondary = GetInventoryItemName( CharacterSecondaryHandSlot:GetID() ); end
	if( main == nil ) then main = ""; end
	if( secondary == nil ) then secondary = ""; end
	GetWeaponSet(index)[0] = main;
	GetWeaponSet(index)[1] = secondary;
end

function RemoveEquipmentSet( set )
	if( GetWeaponSet( set ) ) then
		EquipmentSets[WeaponSet_ServerName][WeaponSet_PlayerName][set] = nil;
	end
end

function CountEquipmentSets()
	local i;
	local count = 0;
	for i = 1, MAX_EQUIPMENT_SETS, 1 do
		if( GetWeaponSet(set) ) then
			if( GetWeaponSet(set)[0] ~= "" or GetWeaponSet(set)[0] ~= "" ) then
				count = count + 1;
			end
		end
	end
end

function ShowEquipmentSet( index, set )
	if( set[0] ~= "" or set[1] ~= "" ) then
		Print( "Equipment set "..index..": "..set[0]..","..set[1] );
	end
end

function ShowEquipmentSets()
	local shown, index, key;
	shown = nil;
	for index, set in EquipmentSets do
		ShowEquipmentSet( index, set );
		shown = 1;
	end
	if( not shown ) then
		Error( "There are no equipment sets to show" );
		return;
	end
end

function NextEquipmentSet()
	if( CountEquipmentSets() == 0 ) then
		Error( "There are no equipment sets specified", 1.0, 0.2, 0.2 );
		return;
	end

	-- do we need to queue this change up?
	if( nil ) then --this doesn't really work, need to figure out why
		if( IsInventoryItemLocked( CharacterMainHandSlot:GetID() ) or
				IsInventoryItemLocked( CharacterSecondaryHandSlot:GetID() ) ) then
			local delayed = function()
				if( IsInventoryItemLocked( CharacterMainHandSlot:GetID() ) or
						IsInventoryItemLocked( CharacterSecondaryHandSlot:GetID() ) ) then
					return false;
				end
				NextEquipmentSet();
				return true;
			end
			table.insert( EquipmentDelayedCommands, 1, delayed );
			return;
		end
	end

	-- figure out which set is currently selected
	local current, next = 1, 2;
	local bags, slots;

	current = DetermineCurrentWeaponSet();
	if( current == nil ) then current = 1; end
	next = current + 1;
	-- is next valid?
	if( next > MAX_EQUIPMENT_SETS ) then next = 1; end
	while( GetWeaponSet(next) == nil or ( GetWeaponSet(next) and GetWeaponSet(next)[0] == "" ) ) do
		next = next + 1;
		if( next > MAX_EQUIPMENT_SETS ) then next = 1; end
		if( next == current ) then break; end
	end

	if( current ~= next and next ~= 0 ) then
		Print( "Switching to equipment set : "..GetWeaponSet(next)[0]..", "..GetWeaponSet(next)[1] );
		EquipWeapons( GetWeaponSet(next)[0], GetWeaponSet(next)[1] );
	end
end

function EquipWeapons( newMain, newSecondary )
	-- item on cursor?
	if( CursorHasItem() ) then
		Error( "There's an item currently on the cursor.  Put it away, please" );
		return;
	end

	if( newMain == nil ) then newMain = ""; end
	if( newSecondary == nil ) then newSecondary = ""; end

	local mainID = CharacterMainHandSlot:GetID();
	local secondaryID = CharacterSecondaryHandSlot:GetID();
	local currentmain = GetInventoryItemName( mainID );
	local currentsecondary = GetInventoryItemName( secondaryID );

	-- handle special cases first
	-- Swapping secondary and main?
	if( newMain == currentsecondary and newSecondary == currentmain ) then
		PickupInventoryItem( secondaryID );
		PickupInventoryItem( mainID );
		AutoEquipCursorItem();
	-- next special case: desired secondary is in main, desired main in bag
	elseif( newSecondary == currentmain and newMain ~= "" ) then
		-- these are hard special cases to handle, and are going to require delaying
		-- a second switch action until the lag delay is complete.
		local source, bag, slot = FindItemByName( newMain );
		if( source == "INVENTORY" or source == "NOT FOUND") then
			Error( "Something has gone very wrong.  Expecing to find "..newMain.." in bags, but found it in inventory instead" );
		else
			-- pickup bag item, move it to main
			PickupContainerItem( bag, slot );
			PickupInventoryItem( mainID );
			local delayed = function()
				local texture, itemCount, locked = GetContainerItemInfo( bag, slot );
				if( IsInventoryItemLocked( secondaryID ) or locked ) then
					return false;
				end
				PickupContainerItem( bag, slot );
				PickupInventoryItem( secondaryID );
				return true;
			end
			table.insert( EquipmentDelayedCommands, 1, delayed );
		end
	-- next special case: desired main is in secondary, desired secondary in bag
	elseif( newMain == currentsecondary and newSecondary ~= "" ) then
		-- these are hard special cases to handle, and are going to require delaying
		-- a second switch action until the lag delay is complete.
		local source, bag, slot = FindItemByName( newSecondary );
		if( source == "INVENTORY" or source == "NOT FOUND" ) then
			Error( "Something has gone very wrong.  Expecing to find "..newSecondary.." in bags, but found it in inventory instead" );
		else
			-- pickup bag item, move it to main
			PickupContainerItem( bag, slot );
			PickupInventoryItem( secondaryID );
			local delayed = function()
				local texture, itemCount, locked = GetContainerItemInfo( bag, slot );
				if( IsInventoryItemLocked( mainID ) or locked ) then
					return false;
				end
				PickupContainerItem( bag, slot );
				PickupInventoryItem( mainID );
				return true;
			end
			table.insert( EquipmentDelayedCommands, 1, delayed );
		end
	else
		-- is the main hand item already equipped?
		if( newMain ~= "" and currentmain ~= newMain ) then
			-- find it in the bag
			local source, bag, slot = FindItemByName( newMain );
			if( source == "INVENTORY" ) then
				-- we should only ever get here if there is no secondary item specified
				-- is it in the offhand?  if so, we need to move it into a bag
				if( newMain == currentsecondary ) then
					PickupInventoryItem( secondaryID );
					PickupInventoryItem( mainID );
					local temp = currentsecondary;
					currentsecondary = currentmain;
					currentmain = temp;
					if( CursorHasItem() ) then
						bag, slot = FindEmptyBagSlot();
						if( not bag ) then
							Error( "Inventory is full!  Switching to these items requires an empty inventory slot" );
							return;
						end
						PickupInventoryContainerItem( bag, slot );
					end
				end
				return;
			elseif( source == "NOT FOUND" ) then
				Error( "Cannot find item "..newMain.." for weapon set" );
				return;
			end
			-- bag, slot is now the location of our new main hand weapon
			PickupContainerItem( bag, slot );
			PickupInventoryItem( mainID );
		end
		-- now secondary hand
		if( newSecondary ~= "" and currentsecondary ~= newSecondary ) then
			-- find it
			local source, bag, slot = FindItemByName( newSecondary );
			if( not bag ) then
				Error( "Cannot find item "..newSecondary.." for weapon set" );
				return;
			end
			-- bag, slot is the location of our new secondary hand weapon
			PickupContainerItem( bag, slot );
			PickupInventoryItem( secondaryID );
		end
	end
end

SLASH_WEAPONSET1 = "/weaponset"
SLASH_WEAPONSET2 = "/ws"
SLASH_WEAPONSET1 = "/weaponsets"
SlashCmdList["WEAPONSET"] = function(msg)
	ToggleWeaponSets();
end

SLASH_EQ1 = "/eqhelp";
SLASH_EQ2 = "/eq";
SlashCmdList["EQ"] = function(msg)
	Print( "**********Equipment Set help************" );
	Print( "/eqhelp or /eq - this message", 1.0, 1.0, 0.2 );
	Print( "/eqadd - adds the current weapon set", 1.0, 1.0, 0.2 );
	Print( "/eqlist - shows all your equipment sets", 1.0, 1.0, 0.2 );
	Print( "/eqremove N - removes equipment set number N", 1.0, 1.0, 0.2 );
end
SLASH_EQADD1 = "/eqadd";
SLASH_EQADD2 = "/eqa";
SLASH_EQLIST1 = "/eqlist";
SLASH_EQLIST2 = "/eql";
SLASH_EQREMOVE1 = "/eqremove";
SLASH_EQREMOVE2 = "/eqr";
SLASH_EQREMOVE3 = "/eqrem";
SlashCmdList["EQADD"] = function(msg)
	AddEquipmentSet();
end
SlashCmdList["EQLIST"] = function(msg)
	ShowEquipmentSets();
end
SlashCmdList["EQREMOVE"] = function(msg)
	local rem = 1;
	if( msg ~= "" ) then
		rem = msg + 0;
	end
	RemoveEquipmentSet( rem );
end